from gtts import gTTS
import subprocess
import openai
import speech_recognition as sr

#openai.api_key = "your-openai-api-key"

openai.api_key = "********************************"

def text_to_speech(text):
    # Use gTTS to convert text to speech and save to an audio file
    tts = gTTS(text=text, lang="en")
    tts.save("output.mp3")
    # Use mpg123 to play the audio file
    subprocess.run(["mpg123", "-q", "output.mp3"])

def record_audio(file_name, duration=10):
    # Record audio from USB microphone using arecord
    command = ["arecord", "-D", "plughw:1,0", "-f", "S16_LE", "-r", "16000", "-d", str(duration), file_name]
    subprocess.run(command)

def speech_to_text(file_name):
    recognizer = sr.Recognizer()

    # Load audio file for recognition
    with sr.AudioFile(file_name) as source:
        audio_data = recognizer.record(source)

    try:
        # Use Google Speech Recognition to convert audio to text
        text = recognizer.recognize_google(audio_data)
        return text.lower()  # Convert to lowercase for easier comparison
    except sr.UnknownValueError:
        print("Sorry, could not understand the audio.")
        return ""
    except sr.RequestError as e:
        print(f"Could not request results; {e}")
        return ""

def get_feedback():
    while True:
        feedback = speech_to_text("feedback.wav")
        if "yes" in feedback:
            return True
        elif "no" in feedback:
            return False
        else:
            text_to_speech("Please say 'yes' or 'no' to confirm.")

if __name__ == "__main__":
    while True:
        # Get the question
        question = "Please ask your question:"
        text_to_speech(question)

        # Record audio for question
        record_audio("question.wav", duration=5)  # Increased duration for longer responses
        print("Recording complete. Processing question...")

        spoken_question = speech_to_text("question.wav")
        if spoken_question:
            print(f"Spoken Question: {spoken_question}")
        else:
            print("Failed to recognize the question. Exiting.")
            exit()

        # Speak the question for confirmation
        text_to_speech(f"You asked: {spoken_question}")  # Speak the question for confirmation
        text_to_speech("Is this the correct question? Say 'yes' or 'no'.")  # Ask for confirmation

        # Record feedback
        record_audio("feedback.wav", duration=5)
        feedback = speech_to_text("feedback.wav")
        if "yes" in feedback:
            break  # Exit loop if the question is correct
        else:
            text_to_speech("Let's try again.")  # Speak out for re-asking the question

    # Use OpenAI's API to get an answer
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        temperature=0.8,
        max_tokens=200,    #default 200  500 to 1000 slower response with higher value
        messages=[
            {"role": "system", "content": spoken_question}
        ]
    )

    generated_answer = completion.choices[0].message["content"]
    print(generated_answer)
    text_to_speech(generated_answer)  # Speaking the answer
